import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { CreateExerciseDto } from "../dto/create-exercise.dto";
import { Exercise } from "../models/exercise";
import { BackendError } from "app/shared/models/backend-error";
import { TaskInstance } from "../models/task-instance";
import { FileRef } from "app/features/tasks/models/file-ref";
import { DownloadService } from "app/shared/services/download.service";
import { CourseResult } from "../../courses/models/course-result";
import { UnitAssessmentProgress } from "../models/unit-assessment-progress";

@Injectable({
    providedIn: 'root'
})
export class ExerciseService extends BaseService {
    constructor(private http: HttpClient, private downloadService: DownloadService) {
        super();
    }

    getById(id: number): Observable<Exercise> {
        return this.http.get<Exercise>(`${environment.backendUrl}/exercise/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => new Exercise(exercise))
        );
    }

    getDescription(exerciseId: number, raw?: boolean): Observable<string> {
        let params = new HttpParams();
        if(raw != null) {
            params = params.set('raw', raw);
        }

        return this.http.get(`${environment.backendUrl}/exercise/${exerciseId}/description`, { 
            responseType: 'text', 
            params: params
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    getTaskInstance(exerciseId: number, taskId: number): Observable<TaskInstance> {
        return this.http.get<TaskInstance>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(taskInstance => new TaskInstance(taskInstance))
        );
    }

    getTaskContent(exerciseId: number, taskId: number): Observable<string> {
        return this.http.get(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/content`, { responseType: 'text'}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    create(dto: CreateExerciseDto): Observable<Exercise> {
        return this.http.post<Exercise>(`${environment.backendUrl}/exercise`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => {
                return new Exercise(exercise);
            })
        );
    }

    update(exerciseId: number, dto: CreateExerciseDto, confirmDangerousAction?: boolean): Observable<Exercise> {
        let headers = {};
        if(confirmDangerousAction) {
            headers['ConfirmDangerousAction'] = "true";
        }

        return this.http.put<Exercise>(`${environment.backendUrl}/exercise/${exerciseId}`, dto, {
            headers: headers
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercise => {
                return new Exercise(exercise);
            })
        );
    }

    getForCourse(courseId: number, query?: string): Observable<Exercise[]> {
        let params = new HttpParams();
        params = params.set('courseId', courseId);
        if(query) {
            params = params.set('query', query);
        }
        
        return this.http.get<Exercise[]>(`${environment.backendUrl}/exercise`, {
            params: params,
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(exercises => exercises.map(exercise => new Exercise(exercise)))
        )
    }


    getTaskInstanceFiles(exerciseId: number, taskId: number): Observable<FileRef[]> {
        return this.http.get<FileRef[]>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files`)
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(files => files.map(f => new FileRef(f)))
        );
    }

    getFileUrl(exerciseId: number, taskId: number, ref: FileRef): string {
        return `${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/files/${ref.file.uuid}/download`;
    }

    downloadFile(exerciseId: number, taskId: number, file: FileRef): Observable<void> {
        return this.downloadService.downloadFile(this.getFileUrl(exerciseId, taskId, file));
    }

    downloadBundle(exerciseId: number, taskId: number): Observable<void> {
        return this.downloadService.downloadFile(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/bundle`);
    }

    getFile(exerciseId: number, taskId: number, ref: FileRef): Observable<string> {
        return this.http.get(this.getFileUrl(exerciseId, taskId, ref), { 
            responseType: 'text',
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map((data: any) => data)
        );
    }

    getTaskInstances(exerciseId: number): Observable<TaskInstance[]> {
        return this.http.get<TaskInstance[]>(`${environment.backendUrl}/exercise/${exerciseId}/task`)
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(taskInstances => taskInstances.map(ti => new TaskInstance(ti)))
        );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/exercise/${id}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }

    publishSolution(exerciseId: number, taskId: number, publish: boolean): Observable<void> {
        return this.http.put<void>(`${environment.backendUrl}/exercise/${exerciseId}/task/${taskId}/public`, {
            pub: publish
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    getAssessmentProgress(exerciseId: number, unitId?: number): Observable<UnitAssessmentProgress[]> {
        let params = new HttpParams();
        if(unitId != null) {
            params = params.set('unitId', unitId);
        }

        return this.http.get<UnitAssessmentProgress[]>(`${environment.backendUrl}/exercise/${exerciseId}/assessment`, {
            params: params 
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(assessmentProgresses => assessmentProgresses.map(progress => new UnitAssessmentProgress(progress)))
        )
    }
}