import { Component, OnInit } from '@angular/core';
import { User } from 'app/core/models/user';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { ParticipantAttendance } from 'app/features/attendances/dto/participant-attendance.dto';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, forkJoin } from 'rxjs';
import { takeUntil, flatMap, map, filter, take } from 'rxjs/operators';
import { BackendError } from 'app/shared/models/backend-error';
import { CourseUnitEvent } from 'app/features/courses/models/course-unit-event';
import { Course } from 'app/features/courses/models/course';
import { CourseService } from 'app/features/courses/services/course.service';
import { UserService } from 'app/core/services/user.service';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { AppStateService } from 'app/core/services/app.state.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { ComponentWithOptionalLifecycleHooks } from 'app/shared/route-reuse-life.cycle.directive';
import { CourseResult } from 'app/features/courses/models/course-result';
import { ParamHelper } from 'app/shared/helper/param.helper';

@Component({
  selector: 'ngx-participant-progress',
  templateUrl: './participant-progress.component.html',
  styleUrls: ['./participant-progress.component.scss']
})
export class ParticipantProgressComponent extends BaseComponent implements OnInit, ComponentWithOptionalLifecycleHooks {
  loading: boolean = true;
  user: User;
  principal: SubatoPrincipal;
  userId: number;
  course: Course;
  participantAttendances: ParticipantAttendance[];
  events: CourseUnitEvent[];
  courseResult: CourseResult;
  hideInvisible;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diesen Kurs konnten wir leider nicht finden.'
    }
  };

  detached: boolean = false;

  constructor(private courseService: CourseService,
    private appStateService: AppStateService,
    private userService: UserService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router) {
    super();
  }

  get canSeeFilter() {
    return this.principal != null && this.principal.hasAnyGroup(this.course.id);
  }

  onAttach() {
    this.detached = false;
    this.appStateService.setCourse(this.course);
  }

  onDetach() {
    this.detached = true;
  }

  ngOnInit(): void {
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
        filter(() => !this.detached),
        flatMap(([params, user]) => {
          if(user == null) {
            throw BackendError.accessDenied();
          }

          this.loading = true;
          this.principal = user;
          let courseId = ParamHelper.parseIdOrThrow(params, "course_id");
          let userId = params.has("user_id") ? ParamHelper.parseIdOrThrow(params, "user_id") : user.id;
          this.userId = userId;
          return this.courseService.getById(courseId);   
        }),
        flatMap((course) => {
          this.course = course;
          this.appStateService.setCourse(course);
          this.hideInvisible = this.canSeeFilter;
          return this.initUser(this.userId);
        })
      )
      .subscribe(() => {
        this.loading = false;
      }, (error: any) => {
        if(BackendError.isBackendError(error)) {
            this.error = error;
            this.loading = false;
        } else {
          throw error;
        }
      });
  }

  onHideInvisibleChanged(hideInvisible: boolean) {
    this.hideInvisible = hideInvisible;
    this.courseService.getProgressForUser(this.course.id, this.userId, this.hideInvisible)
      .pipe(take(1))
      .subscribe(courseResult => this.courseResult = courseResult)
  }

  initUser(userId: number) {
    return forkJoin(
      this.courseService.getAttendancesForUser(this.course.id, userId), 
      this.courseService.getProgressForUser(this.course.id, userId, this.hideInvisible),
      this.userService.getById(userId)
    )
    .pipe(
      map(([attendances, courseResult, user]) => {
        this.user = user;
        this.participantAttendances = attendances;
        this.courseResult = courseResult;
      })
    );
  }
}
