import { Component, Input, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { NbDialogService } from '@nebular/theme';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { DialogConfirmationComponent } from 'app/shared/dialogs/dialog-confirmation/dialog-confirmation.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil, filter, flatMap } from 'rxjs/operators';
import { Language } from '../../models/language';
import { LanguageService } from '../../services/language.service';

@Component({
  selector: 'ngx-languages-list',
  templateUrl: './languages-list.component.html',
  styleUrls: ['./languages-list.component.scss']
})
export class LanguagesListComponent extends BaseComponent implements OnInit {
  @Input() principal: SubatoPrincipal;

  languages: Language[] = [];
  loading: boolean = true;
  error: BackendError;
  deletingLanguage: Language | null;

  constructor(
    private languageService: LanguageService,
    private router: Router,
    private dialogService: NbDialogService,

    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.languageService.getAll()
    .subscribe(languages => {
      this.languages = languages;
      this.loading = false;
    }, error => {
      this.loading = false;

      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.notificationService.showError(
          error.format(
            "Fehler beim Laden der Sprachen",
          )
        );
      } else {
        throw error;
      }
    });

  }

  deleteLanguage(lang: Language) {
    this.dialogService.open(DialogConfirmationComponent, {
      context: {
        title: `Löschung von Sprache: ${lang.name}`,
        text: `Bist du wirklich sicher, dass du die Sprache '${lang.name}' löschen willst? Die Sprache wird bei allen verknüpften Aufgaben entfernt. Achte darauf, dass beim Aufgabenimport jetzt ein Fehler entsteht, wenn diese Sprache noch irgendwo referenziert wird.`,
        actionText: 'Löschen'
      }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.confirmed),
      flatMap(() => {
        this.deletingLanguage = lang;
        return this.languageService.delete(lang.id);
      })
    ).subscribe(() => {
      this.deletingLanguage = null;
      let idx = this.languages.indexOf(lang);
      if(idx != -1) {
        this.languages.splice(idx, 1);
      }

      this.notificationService.showSuccess("Sprache erfolgreich gelöscht.");
    }, error => {
      this.deletingLanguage = null;
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler beim Löschen der Sprache",
          )
        );
      } else {
        throw error;
      }
    })
  }
}
