import { Component, Input, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { DataQueryParams } from 'app/shared/models/data-query-params';
import { Page } from 'app/shared/models/page';
import { Subject } from 'rxjs';
import { takeUntil, debounceTime } from 'rxjs/operators';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';
import { Task } from 'app/features/tasks/models/task';

@Component({
  selector: 'ngx-task-pool-tasks',
  templateUrl: './task-pool-tasks.component.html',
  styleUrls: ['./task-pool-tasks.component.scss']
})
export class TaskPoolTasksComponent extends BaseComponent implements OnInit {
  @Input() pool: TaskPool;
  tasks: Task[] = [];

  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  page: Page<Task> | null;
  loadingPage = false;

  loading: boolean = true;
  error: BackendError;

  constructor(
    private taskPoolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.taskPoolService.getTasks(this.pool.id)
      .pipe(
        takeUntil(this.unsubscribe)
      )
      .subscribe(tasks => {
        this.tasks = tasks.content;
        this.page = tasks;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });

    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.query = query;
        this.page = null;
        this.loadNextTasks(true);
      });
  }

  loadNextTasks(reset: boolean = false) {
    if (this.loadingPage || (this.page != null && !this.page.hasNext())) {
      return;
    }

    this.loadingPage = true;

    let projection = DataQueryParams.next(this.page);
    this.taskPoolService.getTasks(this.pool.id, { q: this.query, projection: projection })
      .subscribe(page => {
        this.page = page;
        if (reset) {
          this.tasks = [];
        }
        this.tasks.push(...page.content);
        this.loadingPage = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loadingPage = false;
        } else {
          throw error;
        }
      });
  }

  filterTasks(query) {
    this.queryChanged.next(query);
  }

  public refresh() {
    this.page = null;
    this.loadNextTasks(true);
  }
}
