import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { environment } from "environments/environment";
import { Observable, of } from "rxjs";
import { Task } from "app/features/tasks/models/task";
import { catchError, flatMap, map } from "rxjs/operators";
import { TaskFeedback } from "../models/task-feedback";
import { BaseService } from "app/shared/services/base.service";
import { CreateTaskFeedbackDto } from "../dto/create-task-feedback.dto";
import { BackendError } from "app/shared/models/backend-error";
import { FileRef } from "../models/file-ref";
import { DataQueryParams } from "app/shared/models/data-query-params";
import { Page } from "app/shared/models/page";
import { DownloadService } from "app/shared/services/download.service";

export interface TaskParams {
    q?: string;
    poolId?: number;
    projection?: DataQueryParams;
}

@Injectable({
    providedIn: 'root'
})
export class TaskService extends BaseService {
    constructor(private http: HttpClient, private downloadService: DownloadService) {
        super();
    }

    getAll(params?: TaskParams): Observable<Page<Task>> {
        let httpParams = DataQueryParams.toHttpParams(params?.projection);

        if(params?.q) {
            httpParams = httpParams.set('q', params.q);
        }

        if(params?.poolId) {
            httpParams = httpParams.set('poolId', params.poolId);
        }

        return this.http.get<Page<Task>>(`${environment.backendUrl}/task`, { 
            params: httpParams,
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(page => new Page<Task>(page, item => new Task(item)))
        )
    }

    getById(id: number): Observable<Task> {
        return this.http.get<Task>(`${environment.backendUrl}/task/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(task => new Task(task))
        );
    }

    getContent(id: number): Observable<string> {
        return this.http.get(`${environment.backendUrl}/task/${id}/content`, { responseType: 'text'}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    getExampleSpecUrl(): string {
        return `${environment.backendUrl}/00004Appointment.zip`;
    }

    getSpecUrl(taskId: number): string {
        return `${environment.backendUrl}/task/${taskId}/spec`;
    }

    downloadSpec(taskId: number): Observable<void> {
        return this.downloadService.downloadFile(this.getSpecUrl(taskId));
    }

    downloadBundle(taskId: number): Observable<void> {
        return this.downloadService.downloadFile(`${environment.backendUrl}/task/${taskId}/bundle`);
    }

    createFeedback(id: number, dto: CreateTaskFeedbackDto): Observable<TaskFeedback> {
        return this.http.put<TaskFeedback>(`${environment.backendUrl}/task/${id}/feedback`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    markFeedbackAsRead(id: number, userId: number): Observable<TaskFeedback> {
        return this.http.put<TaskFeedback>(`${environment.backendUrl}/task/${id}/feedback/${userId}/read`, {}).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
        );
    }

    getFeedbackByUser(id: number, userId: number): Observable<TaskFeedback> {
        return this.http.get<TaskFeedback>(`${environment.backendUrl}/task/${id}/feedback/${userId}`)
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(feedback => feedback ? new TaskFeedback(feedback) : null)
        );
    }

    getFeedback(id: number): Observable<TaskFeedback[]> {
        return this.http.get<TaskFeedback[]>(`${environment.backendUrl}/task/${id}/feedback`)
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(feedback => feedback.map(f => new TaskFeedback(f)))
        );
    }

    getFiles(id: number): Observable<FileRef[]> {
        return this.http.get<FileRef[]>(`${environment.backendUrl}/task/${id}/files`)
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(files => files.map(f => new FileRef(f)))
        );
    }

    getFileUrl(taskId: number, ref: FileRef): string {
        return `${environment.backendUrl}/task/${taskId}/files/${ref.file.uuid}/download`;
    }

    downloadFile(taskId: number, file: FileRef): Observable<void> {
        return this.downloadService.downloadFile(this.getFileUrl(taskId, file));
    }

    getFile(taskId: number, ref: FileRef): Observable<string> {
        return this.http.get(this.getFileUrl(taskId, ref), { 
            responseType: 'text',
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map((data: any) => data)
        );
    }

    archive(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/task/${id}`)
            .pipe(
                catchError(e => {
                    throw BackendError.parse(e);
                }),
            );
    }
}