package de.hsrm.sls.subato.aor.term.imp.unit;

import de.hsrm.sls.subato.aor.term.model.ModulePartInstance;
import de.hsrm.sls.subato.aor.term.model.TermData;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseRepository;
import de.hsrm.sls.subato.courses.unit.CourseUnitRepository;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.CreateCourseUnitRequestBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional
public class CourseUnitImportService {
    @Autowired
    CourseUnitRepository courseUnitRepository;

    @Autowired
    LecturerImportService lecturerImportService;

    @Autowired
    ParticipantImportService participantImportService;

    @Autowired
    CourseUnitEventImportService courseUnitEventImportService;

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    CourseUnitService courseUnitService;

    public void importUnits(Course course, TermData termData) {
        course = clearUnits(course);

        var partInstances = getModulePartInstances(termData, course);
        var groupedPartInstances = groupPartInstances(partInstances);

        for (var entry : groupedPartInstances.entrySet()) {
            var modulePartInstances = entry.getValue();

            if (modulePartInstances.size() == 0) {
                continue;
            }

            var courseType = new CourseUnitTypeConverter().convert(entry.getKey().courseType());
            if (courseType == null) {
                continue;
            }

            var hasStudents = modulePartInstances.stream().anyMatch(mpi -> mpi.getStudents().size() != 0);
            if (!hasStudents) {
                continue;
            }

            var leadingModulePartInstance = modulePartInstances.get(0);

            var lecturers = lecturerImportService.importLecturers(leadingModulePartInstance);
            var participants = participantImportService.importParticipants(modulePartInstances);

            var group = entry.getKey().group().isEmpty() ? "A" : entry.getKey().group();

            var request = new CreateCourseUnitRequestBuilder()
                    .setCourse(course)
                    .setName(group)
                    .setAorId(leadingModulePartInstance.getId())
                    .setType(courseType)
                    .setLecturers(lecturers)
                    .setParticipants(participants)
                    .build();

            var unit = courseUnitService.create(request);
            courseUnitEventImportService.importEvents(unit, leadingModulePartInstance);
        }
    }

    private Map<ModulePartInstance.Group, List<ModulePartInstance>> groupPartInstances(ArrayList<ModulePartInstance> partInstances) {
        Map<ModulePartInstance.Group, List<ModulePartInstance>> groupedPartInstances =
                partInstances.stream().collect(
                        Collectors.groupingBy(p ->
                                new ModulePartInstance.Group(p.getGroup(), p.getLecturers(), p.getModulePart().getCourseType())
                        )
                );
        return groupedPartInstances;
    }

    private ArrayList<ModulePartInstance> getModulePartInstances(TermData termData, Course course) {
        var partInstances = new ArrayList<ModulePartInstance>();
        for (var programModule : course.getModules()) {
            var aorModuleInstance = termData.getModuleInstances().stream().filter(
                    aorMi -> aorMi.getModuleNodeId().equals(programModule.getAorId())
            ).findFirst().orElse(null);

            if (aorModuleInstance == null) {
                // modul nicht im aktuellen semester vorhanden, sollte eigentlich nicht passieren
                continue;
            }

            partInstances.addAll(aorModuleInstance.getModulePartInstances());
        }
        return partInstances;
    }

    private Course clearUnits(Course course) {
        var units = course.getUnits().stream().filter(u -> u.getType() != CourseUnitType.Misc).collect(Collectors.toList());
        course.getUnits().removeAll(units);
        course = courseRepository.save(course);
        return course;
    }
}
