package de.hsrm.sls.subato.courses.course;

import de.hsrm.sls.subato.attendances.attendance.AttendanceService;
import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceDetailDto;
import de.hsrm.sls.subato.attendances.attendance.dto.ParticipantAttendanceMapper;
import de.hsrm.sls.subato.auth.policy.attendance.AttendancePermission;
import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.file.CourseFileDto;
import de.hsrm.sls.subato.courses.file.CourseFileMapper;
import de.hsrm.sls.subato.courses.membership.*;
import de.hsrm.sls.subato.courses.unit.CourseUnitDto;
import de.hsrm.sls.subato.courses.unit.CourseUnitMapper;
import de.hsrm.sls.subato.courses.unit.CourseUnitService;
import de.hsrm.sls.subato.courses.result.CourseResultDto;
import de.hsrm.sls.subato.courses.result.CourseResultMapper;
import de.hsrm.sls.subato.courses.result.CourseResultService;
import de.hsrm.sls.subato.modules.term.TermNotFoundException;
import de.hsrm.sls.subato.shared.markdown.MarkdownParser;
import de.hsrm.sls.subato.user.UserDetailDto;
import de.hsrm.sls.subato.user.UserMapper;
import de.hsrm.sls.subato.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseController {
    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private CourseStaffMembershipMapper courseStaffMembershipMapper;

    @Autowired
    private CourseUnitMapper courseUnitMapper;

    @Autowired
    private CourseService courseService;

    @Autowired
    private CourseUnitService courseUnitService;

    @Autowired
    private PrincipalService principalService;

    @Autowired
    private CourseFileMapper courseFileMapper;

    @Autowired
    private PolicyEvaluator policy;

    @Autowired
    private UserService userService;

    @Autowired
    private CourseStaffMembershipService staffMembershipService;

    @Autowired
    private CourseResultService courseResultService;

    @Autowired
    private AttendanceService attendanceService;

    @Autowired
    private CourseResultMapper courseResultMapper;

    @Autowired
    private ParticipantAttendanceMapper participantAttendanceMapper;

    @Autowired
    UserMapper userMapper;

    @Autowired
    private MarkdownParser markdownParser;

    @GetMapping("/course/{id}")
    @Operation(summary = "Liefert Metadaten zu einem Kurs")
    CourseDto getOne(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW));

            return courseMapper.mapDetail(course);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{id}/description")
    @Operation(summary = "Liefert die Kursbeschreibung (entweder in Rohform als Markdown oder gerendert als HTML)")
    String getDescription(@PathVariable Long id,
                          @Parameter(description = "false wenn nicht angegeben") @RequestParam Optional<Boolean> raw) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW));
            return raw.orElse(false) ? course.getDescription() : markdownParser.convertToHtml(course.getDescription());
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course")
    @Operation(summary = "Liefert Metadaten zu allen Kursen")
    List<CourseDto> getAll(@RequestParam Optional<Long> termId, @RequestParam("q") Optional<String> query, @PageableDefault Pageable page) {
        var filter = new CourseFilter()
                .termId(termId.orElse(null))
                .query(query.orElse(null));
        var courses = courseService.findAll(filter, CoursesAction.COURSE_VIEW, page.getSort());

        return courses.stream()
                .map(c -> courseMapper.mapDetail(c))
                .collect(Collectors.toList());
    }

    @PostMapping("/course")
    public CourseDto create(@RequestBody CreateCourseDto dto) {
        try {
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(null, CoursesAction.COURSE_CREATE));
            var request = new CreateCourseRequestBuilder()
                    .setName(dto.getName())
                    .setShortName(dto.getShortName())
                    .setCourseId(dto.getCourseId())
                    .setTermId(dto.getTermId())
                    .setShowEmails(true)
                    .setModuleIds(dto.getModuleIds()).build();

            var course = courseService.create(request, principalService.getPrincipal());
            return courseMapper.mapDetail(course);
        } catch(TermNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, TermNotFoundException.REASON);
        }
    }

    @PutMapping ("/course/{id}")
    CourseDto update(@PathVariable Long id, @RequestBody UpdateCourseDto dto) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_UPDATE));

            course = courseService.update(course, dto);
            return courseMapper.mapDetail(course);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, "Der Kurs existiert nicht");
        }
    }

    @DeleteMapping("/course/{id}")
    public void deleteCourse(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_DELETE));

            courseService.delete(course);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{id}/units")
    List<CourseUnitDto> getUnits(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW));

            var units = courseUnitService.findByCourse(course);
            return units.stream().map(unit -> courseUnitMapper.map(unit)).toList();
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{id}/files")
    List<CourseFileDto> getFiles(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW));

            var files = course.getFiles();
            return files.stream().map(file -> courseFileMapper.map(file)).toList();
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{id}/staff")
    List<CourseStaffMembershipDetailDto> getStaff(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW));

            var memberships = course.getMemberships();
            return memberships.stream().map(membership -> courseStaffMembershipMapper.mapDetail(membership)).toList();
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{id}/participants")
    List<UserDetailDto> getParticipants(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_VIEW_PARTICIPANTS));

            var participants = courseService.getParticipants(course);
            return participants.stream().map(u -> userMapper.mapDetail(u)).collect(Collectors.toList());
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @DeleteMapping("/course/{id}/staff/{userId}/group/{group}")
    public void removeStaff(@PathVariable Long id, @PathVariable Long userId, @PathVariable StaffGroup group) {
        try {
            var course = courseService.findById(id);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_UPDATE));

            staffMembershipService.removeMembership(course, user, group);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        } catch(CourseStaffMembershipNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseStaffMembershipNotFoundException.REASON);
        }
    }

    @PutMapping("/course/{id}/staff/{userId}/group/{group}")
    public CourseStaffMembershipDetailDto addStaff(@PathVariable Long id, @PathVariable Long userId, @PathVariable StaffGroup group) {
        try {
            var course = courseService.findById(id);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_UPDATE));

            var membership = staffMembershipService.addMembership(course, user, group);
            return courseStaffMembershipMapper.mapDetail(membership);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        } catch(CourseStaffMembershipAlreadyExistsException ex) {
            throw new ResponseStatusException(BAD_REQUEST, CourseStaffMembershipAlreadyExistsException.REASON);
        }
    }

    @PostMapping("/course/{id}/reimport")
    public CourseDto reimport(@PathVariable Long id) {
        try {
            var course = courseService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(course, CoursesAction.COURSE_UPDATE));

            course = courseService.reimport(course);
            return courseMapper.mapDetail(course);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{courseId}/result/{userId}")
    @Operation(summary = "Liefert Informationen zum Fortschritt (Zusammenfassung) eines Benutzers über alle Übungen eines Kurses hineweg")
    CourseResultDto getProgressForUser(@PathVariable Long courseId, @PathVariable Long userId, @RequestParam Optional<Boolean> hideInvisibleExercises) {
        try {
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(courseId, userId), CoursesAction.COURSE_PROGRESS_VIEW));

            var courseResult = courseResultService.generateFor(courseId, userId, hideInvisibleExercises.orElse(true));
            return courseResultMapper.map(courseResult);
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, CourseNotFoundException.REASON);
        }
    }

    @GetMapping("/course/{courseId}/attendance/{userId}")
    @Operation(summary = "Zusammenfassung der Anwesenheitsinformationen des Benutzers in allen Kurseinheiten (z.B. Praktikumsgruppen)")
    List<ParticipantAttendanceDetailDto> getCourseAttendance(@PathVariable Long courseId, @PathVariable Long userId) {
        try {
            var course = courseService.findById(courseId);
            var user = userService.findById(userId);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(course, user), AttendancePermission.ATTENDANCE_VIEW));

            var courseUnitAttendance = attendanceService.getAttendanceForCourseAndUser(course, user);
            return courseUnitAttendance.stream()
                    .map(participantAttendance -> participantAttendanceMapper.map(participantAttendance))
                    .toList();
        } catch(CourseNotFoundException ex) {
            throw new ResponseStatusException(BAD_REQUEST, "Kurs existiert nicht");
        }
    }
}
