package de.hsrm.sls.subato.solutions.solution;

import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.solutions.comment.SolutionComment;
import de.hsrm.sls.subato.solutions.evaluation.result.EvalResult;
import de.hsrm.sls.subato.exercises.submission.Submission;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;

import jakarta.annotation.Nullable;
import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a complex solution for a {@link de.hsrm.sls.subato.tasks.task.Task}
 * in an {@link de.hsrm.sls.subato.exercises.exercise.Exercise}
 * (represented by {@link TaskInstance}), that may be evaluated automatically
 */
@Entity
@Table(name = "solution")
public class Solution {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @ManyToOne
    private Task task;

    @ManyToOne
    private Exercise exercise;

    @ManyToOne
    private User submittedBy;

    @Column(columnDefinition = "TIMESTAMP")
    @NotNull
    private LocalDateTime submittedAt;

    /**
     * internal reference, indicating if the solution is from the legacy system.
     * if it is, then the solution might be inconsistent. not all fields may be present
     */
    @Column(unique = true)
    Long sourceId;

    /**
     * results of the evaluation.
     *
     * will be null if tests were never run (because of missing evaluator)
     * or if they could not be executed (i.e. because of compiler errors)
     */
    @OneToOne(cascade = CascadeType.ALL)
    @Nullable
    private EvalResult evalResult;

    /**
     * submission, this solution belongs to.
     *
     * null if
     * 1.) the solution was submitted after deadline or
     * 2.) the solution belongs to an exercise for which checking/grading is disabled
     */
    @ManyToOne
    @Nullable
    private Submission submission;

    /**
     * if the solution was submitted after deadline
     * and the user explicitly stated to show the solution to the lecturers
     */
    private boolean delayed;

    /**
     * if this solution exceeded the value at {@link Task#attempts}
     */
    private boolean attemptsExceeded;

    /**
     * how was this solution submitted?
     */
    @NotNull
    private Client submittedWith;

    @OneToMany(mappedBy = "solution", cascade = { CascadeType.ALL })
    private List<SolutionComment> comments = new ArrayList<>();

    public Solution() { }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public Long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public Exercise getExercise() {
        return exercise;
    }

    public void setExercise(Exercise exercise) {
        this.exercise = exercise;
    }


    public TaskInstance getTaskInstance() {
        return exercise.getTaskInstances().stream().filter(ti -> ti.getTask().getId() == task.getId()).findFirst().orElse(null);
    }

    public User getSubmittedBy() {
        return submittedBy;
    }

    @Nullable
    public EvalResult getEvalResult() {
        return evalResult;
    }

    public void setEvalResult(@Nullable EvalResult evalResult) {
        this.evalResult = evalResult;
    }

    public void setSubmittedBy(User creator) {
        this.submittedBy = creator;
    }

    public LocalDateTime getSubmittedAt() {
        return submittedAt;
    }

    public void setSubmittedAt(LocalDateTime submittedAt) {
        this.submittedAt = submittedAt;
    }

    public Submission getSubmission() {
        return submission;
    }

    public void setSubmission(Submission submission) {
        this.submission = submission;
    }

    public boolean isDelayed() {
        return delayed;
    }

    public void setDelayed(boolean delayed) {
        this.delayed = delayed;
    }

    public boolean isAttemptsExceeded() {
        return attemptsExceeded;
    }

    public void setAttemptsExceeded(boolean attemptsExceeded) {
        this.attemptsExceeded = attemptsExceeded;
    }

    public List<SolutionComment> getComments() {
        return comments;
    }

    public void setComments(List<SolutionComment> comments) {
        this.comments = comments;
    }

    public Client getSubmittedWith() {
        return submittedWith;
    }

    public void setSubmittedWith(Client submittedWith) {
        this.submittedWith = submittedWith;
    }
}
