package de.hsrm.sls.subato.solutions.submit;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.solutions.SolutionsAction;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.solutions.evaluation.EvaluationFailedException;
import de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException;
import de.hsrm.sls.subato.solutions.evaluation.SolutionEvaluationService;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.solution.SolutionRepository;
import de.hsrm.sls.subato.user.UserService;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Validation;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * handles the submission of a solution to a task, which will occur if lecturers are testing their tasks
 * with sample solutions. The solutions have to be persisted in order to be able to automatically evaluate them.
 *
 * Solutions will not be cleaned up automatically, they remain in the system
 * to ensure that solutions can be accessed later for debugging purposes
 */
@Service
public class TaskSubmitHandler implements SubmitHandler {
    @Autowired
    private SolutionRepository solutionRepository;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    @Autowired
    private UserService userService;

    @Autowired
    private SolutionEvaluationService solutionEvaluationService;

    @Autowired
    private PolicyEvaluator policy;

    /**
     * @throws EvaluationFailedException if the evaluation of the solution failed
     * @throws InvalidSolutionException if the solution cannot be evaluated because it was incorrectly submitted
     */
    @Override
    public Solution handle(SubmitSolutionRequest request, SubatoPrincipal principal, LocalDateTime submittedAt) {
        policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(Pair.of(request.exercise(), request.task()), SolutionsAction.SOLUTION_SUBMIT_TASK));
        var user = principal != null ? userService.findById(principal.getId()) : null;

        Solution solution = new Solution();
        solution.setSubmittedBy(user);
        solution.setTask(request.task());
        solution.setSubmittedAt(submittedAt);
        solution.setSubmittedWith(request.client());

        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(solution);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }

        solution = solutionRepository.save(solution);

        solutionContentRepository.store(request.content(), solution);

        var task = solution.getTask();
        if(task.evaluationPossible()) {
            var evalResult = solutionEvaluationService.test(solution);
            solution.setEvalResult(evalResult);
            return solutionRepository.save(solution);
        }

        return solution;
    }
}
