package de.hsrm.sls.subato.tasks.stef.parser.meta;

import com.google.common.base.Strings;
import de.hsrm.sls.subato.tasks.stef.spec.*;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModeXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModesXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import de.hsrm.sls.subato.tasks.task.SubmissionModeType;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SubmissionModeExtractor {
    public void extract(TaskXml doc, Meta meta) {
        SubmissionModeConfig config;

        if (doc.getSubmissionModes() != null) {
            if (doc.getSubmissionModes().getSubmissionModes().size() == 0) {
                throw new InvalidSpecException("Wenn %s konfiguriert wird, muss mind. ein %s konfiguriert werden".formatted(TaskXml.SUBMISSION_MODES, SubmissionModesXml.SUBMISSION_MODE));
            }

            var modes = new ArrayList<SubmissionMode>();
            for (var xmlSubmissionMode : doc.getSubmissionModes().getSubmissionModes()) {
                var submissionMode = extractSubmissionMode(doc, meta, xmlSubmissionMode);
                modes.add(submissionMode);
            }

            SubmissionModeType defaultMode = extractDefaultMode(doc, modes);
            if(defaultMode == null) {
                defaultMode = getDefaultMode(modes);
            }

            config = new SubmissionModeConfig(modes, defaultMode);
        } else {
            config = configureDefaultSubmissionModes(doc, meta);
        }

        meta.setSubmissionModeConfig(config);
        addDefaultFileExtensionIfRequired(config, meta);
    }

    private SubmissionModeConfig configureDefaultSubmissionModes(TaskXml doc, Meta meta) {
        var modes = new ArrayList<SubmissionMode>();

        var fileSubmissionMode = new FileSubmissionMode();
        fileSubmissionMode.setAllowArchives(getAllowArchivesDefaultValue());
        fileSubmissionMode.setExtractArchives(getExtractArchivesValue(doc));
        fileSubmissionMode.setExtensions(getDefaultExtensions(meta));
        modes.add(fileSubmissionMode);

        var templateFilesCount = meta.getTemplateFiles().size();
        if (templateFilesCount == 1) {
            modes.add(new TextSubmissionMode());
        }

        var config = new SubmissionModeConfig(modes, getDefaultMode(modes));
        return config;
    }

    private SubmissionMode extractSubmissionMode(TaskXml doc, Meta meta, SubmissionModeXml xmlSubmissionMode) {
        if (xmlSubmissionMode.getName() == null) {
            throw new InvalidSpecException("%s-Attribut in %s fehlt".formatted(SubmissionModeXml.NAME, SubmissionModesXml.SUBMISSION_MODE));
        }

        SubmissionModeType type;
        try {
            type = SubmissionModeType.valueOf(xmlSubmissionMode.getName());
        } catch (IllegalArgumentException ex) {
            throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(xmlSubmissionMode.getName()));
        }

        switch(type) {
            case FILE: {
                var submissionMode = new FileSubmissionMode();

                var allowArchives = xmlSubmissionMode.getArchives() != null ? xmlSubmissionMode.getArchives().booleanValue() : getAllowArchivesDefaultValue();
                submissionMode.setAllowArchives(allowArchives);

                var extractArchives = xmlSubmissionMode.getExtract() != null ? xmlSubmissionMode.getExtract().booleanValue() : getExtractArchivesValue(doc);
                if(extractArchives && !allowArchives) {
                    throw new InvalidSpecException("%s kann nicht true sein wenn %s false ist".formatted(SubmissionModeXml.EXTRACT, SubmissionModeXml.ARCHIVES));
                }
                submissionMode.setExtractArchives(extractArchives);

                var extensionsString = xmlSubmissionMode.getExtensions();
                if (!Strings.isNullOrEmpty(extensionsString)) {
                    var extensions = Arrays.stream(extensionsString.split(";")).toList();
                    submissionMode.setExtensions(extensions);
                } else {
                    submissionMode.setExtensions(getDefaultExtensions(meta));
                }

                return submissionMode;
            }
            case TEXT: {
                var extensionsString = xmlSubmissionMode.getExtensions();
                if(type == SubmissionModeType.TEXT && !Strings.isNullOrEmpty(extensionsString)) {
                    throw new InvalidSpecException("Angabe von %s für %s %s nicht möglich".formatted(SubmissionModeXml.EXTENSIONS, SubmissionModesXml.SUBMISSION_MODE, SubmissionModeType.TEXT));
                }

                if(type == SubmissionModeType.TEXT && xmlSubmissionMode.getArchives() != null) {
                    throw new InvalidSpecException("Angabe von %s für %s %s nicht möglich".formatted(SubmissionModeXml.ARCHIVES, SubmissionModesXml.SUBMISSION_MODE, SubmissionModeType.TEXT));
                }

                if(type == SubmissionModeType.TEXT && xmlSubmissionMode.getExtract() != null) {
                    throw new InvalidSpecException("Angabe von %s für %s %s nicht möglich".formatted(SubmissionModeXml.EXTRACT, SubmissionModesXml.SUBMISSION_MODE, SubmissionModeType.TEXT));
                }

                var submissionMode = new TextSubmissionMode();
                return submissionMode;
            }
            default: {
                throw new RuntimeException("submissionMode %s not handled".formatted(type));
            }
        }
    }

    private SubmissionModeType extractDefaultMode(TaskXml doc, List<SubmissionMode> submissionModes) {
        SubmissionModeType defaultMode = null;
        if (doc.getSubmissionModes().getDefaultMode() != null) {
            try {
                defaultMode = SubmissionModeType.valueOf(doc.getSubmissionModes().getDefaultMode());

                var isDefaultModeConfigured = submissionModes.stream().map(m -> m.getType()).toList().contains(defaultMode);
                if (!isDefaultModeConfigured) {
                    throw new InvalidSpecException("SubmissionMode %s kann nicht als default verwendet werden, da er nicht konfiguriert wurde".formatted(doc.getSubmissionModes().getDefaultMode()));
                }
            } catch (IllegalArgumentException ex) {
                throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(doc.getSubmissionModes().getDefaultMode()));
            }
        }
        return defaultMode;
    }

    private SubmissionModeType getDefaultMode(List<SubmissionMode> modes) {
        return modes.stream().map(m -> m.getType()).toList().contains(SubmissionModeType.TEXT)
                ? SubmissionModeType.TEXT
                : SubmissionModeType.FILE;
    }

    private boolean getAllowArchivesDefaultValue() {
        return true;
    }

    private boolean getExtractArchivesValue(TaskXml doc) {
        return !Strings.isNullOrEmpty(doc.getEvaluator());
    }

    private List<String> getDefaultExtensions(Meta meta) {
        var extensions = new ArrayList<String>();

        var templateExtensions = meta.getTemplateFiles()
                .stream()
                .map(f -> f.getFile().getExtension())
                .collect(Collectors.toList());

        extensions.addAll(templateExtensions);
        return extensions;
    }

    private void addDefaultFileExtensionIfRequired(SubmissionModeConfig submissionModeConfig, Meta meta) {
        var fileSubmissionMode = submissionModeConfig.getFileSubmissionMode();
        var textSubmissionMode = submissionModeConfig.getTextSubmissionMode();
        var templateFilesCount = meta.getTemplateFiles().size();

        if(submissionModeConfig != null && textSubmissionMode != null && templateFilesCount == 0) {
            if(!fileSubmissionMode.getExtensions().contains(FileRef.DEFAULT_FILE_EXT)) {
                fileSubmissionMode.getExtensions().add(FileRef.DEFAULT_FILE_EXT);
            }
        }
    }
}
