package de.hsrm.sls.subato.aor.term;

import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.unit.CourseUnitType;
import de.hsrm.sls.subato.courses.unit.Weekday;
import org.junit.Assert;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.stream.Collectors;

public class CourseImportAssert {
    record Variables(int lectureAParticipants, int labAParticipants, int labBParticipants, int tutAParticipants) {
    }

    /**
     * check if a simple course with exactly one module is imported correctly
     * @param course
     */
    public static void assertSimpleCourse(Course course) {
        assertCourse(course, new Variables(3, 1, 2, 3));
    }

    /**
     * check if two modules, which were planned together, are merged correctly into one course
     */
    public static void assertModGroupCourse(Course course) {
        // lectureAParticipants: 4 (3 from the first module + 1 from the second module) etc.
        assertCourse(course, new Variables(4, 2, 2, 4));
    }

    /**
     * same as {@link #assertModGroupCourse(Course)} but there are two different Lab A's  (different lecturers)
     * @param course
     */
    public static void assertModGroupPartialCourse(Course course) {
        Assert.assertEquals(6 + 1, course.getUnits().size());

        var labAUnits = course.getUnits().stream().filter(u -> u.getType() == CourseUnitType.Lab && u.getName().equals("A")).collect(Collectors.toList());
        var first = labAUnits.get(0);
        var second = labAUnits.get(1);

        Assert.assertEquals(1, first.getParticipants().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(1, second.getParticipants().stream().map(p -> p.getLoginName()).toList().size());
    }

    private static void assertCourse(Course course, Variables var) {
        var units = course.getUnits();
        Assert.assertEquals(6, units.size());

        var lectureA = course.getUnits().stream().filter(u -> u.getType() == CourseUnitType.Lecture && u.getName().equals("A")).findFirst().get();
        var labA = course.getUnits().stream().filter(u -> u.getType() == CourseUnitType.Lab && u.getName().equals("A")).findFirst().get();
        var labB = course.getUnits().stream().filter(u -> u.getType() == CourseUnitType.Lab && u.getName().equals("B")).findFirst().get();
        var tutA = course.getUnits().stream().filter(u -> u.getType() == CourseUnitType.Tut && u.getName().equals("A")).findFirst().get();

        Assert.assertEquals(var.lectureAParticipants, lectureA.getParticipants().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(var.labAParticipants, labA.getParticipants().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(var.labBParticipants, labB.getParticipants().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(var.tutAParticipants, tutA.getParticipants().stream().map(p -> p.getLoginName()).toList().size());

        Assert.assertEquals(1, lectureA.getLecturers().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(1, labA.getLecturers().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(1, labB.getLecturers().stream().map(p -> p.getLoginName()).toList().size());
        Assert.assertEquals(1, tutA.getLecturers().stream().map(p -> p.getLoginName()).toList().size());

        Assert.assertEquals(2, lectureA.getEventDefinitions().size());
        Assert.assertEquals("B002", lectureA.getEventDefinitions().get(0).getRoom());
        Assert.assertEquals(Weekday.Tuesday, lectureA.getEventDefinitions().get(0).getWeekday());
        Assert.assertEquals(LocalTime.of(11, 45, 0), lectureA.getEventDefinitions().get(0).getTime());
        Assert.assertEquals("B002", lectureA.getEventDefinitions().get(1).getRoom());
        Assert.assertEquals(LocalTime.of(14, 15, 0), lectureA.getEventDefinitions().get(1).getTime());
        Assert.assertEquals(Weekday.Tuesday, lectureA.getEventDefinitions().get(1).getWeekday());

        Assert.assertEquals(60, lectureA.getEvents().size());
        Assert.assertEquals(LocalDateTime.of(2022, 10, 18, 11, 45, 0), lectureA.getEvents().get(0).getTakesPlaceAt());

        Assert.assertEquals(1, course.getAdmins().size());
        Assert.assertEquals(2, course.getLecturers().size());
        Assert.assertEquals(1, course.getTutors().size());
    }
}
