package de.hsrm.sls.subato.policy;

import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipalMapper;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.courses.membership.StaffGroup;
import de.hsrm.sls.subato.courses.unit.CourseUnit;
import de.hsrm.sls.subato.utils.SubatoPrincipalBuilder;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.HashMap;
import java.util.List;

/**
 * Unit-Test für {@link de.hsrm.sls.subato.auth.policy.courses.CourseUpdatePolicy}
 */
@SpringBootTest
@Tag(Tags.UNIT)
public class CourseUpdateAuthorizationTest {
    @Autowired
    SubatoPrincipalMapper mapper;

    @Autowired
    PolicyEvaluator evaluator;

    @MockBean
    CourseService courseService;

    private Course course;

    @BeforeEach
    public void setup() {
        course = new Course();
        course.setId(1L);
        course.setUnits(List.of(new CourseUnit()));

        Mockito.when(courseService.courseExists(1)).thenReturn(true);
    }

    @Test
    public void shouldAllowCourseAdmins() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.ADMIN).build(mapper);

        var req = new CheckPolicyRequest(principal, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(true, evaluator.check(req));
    }

    @Test
    public void shouldDenyCourseLecturers() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.LECTURER).build(mapper);

        var req = new CheckPolicyRequest(principal, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(false, evaluator.check(req));
    }

    @Test
    public void shouldDenyCourseTutors() {
        var principal = new SubatoPrincipalBuilder().membership(course, StaffGroup.TUTOR).build(mapper);

        var req = new CheckPolicyRequest(principal, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(false, evaluator.check(req));
    }

    @Test
    public void shouldDenyParticipants() {
        var principal = new SubatoPrincipalBuilder().participantOf(course).build(mapper);

        var req = new CheckPolicyRequest(principal, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(false, evaluator.check(req));
    }

    @Test
    public void shouldDenyGuests() {
        var req = new CheckPolicyRequest(null, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(false, evaluator.check(req));
    }

    @Test
    public void shouldDenyAllRoles() {
        for(var role : Role.regularRoles()) {
            var principal = new SubatoPrincipalBuilder().role(role).build(mapper);

            var req = new CheckPolicyRequest(principal, course, CoursesAction.COURSE_UPDATE, new HashMap<>());
            Assert.assertEquals("Role %s".formatted(role),false, evaluator.check(req));
        }
    }

    @Test
    public void shouldAllowIfCourseNotExists() {
        for(var group : StaffGroup.values()) {
            var principal = new SubatoPrincipalBuilder().membership(course, group).build(mapper);

            var req = new CheckPolicyRequest(principal, -1L, CoursesAction.COURSE_UPDATE, new HashMap<>());
            Assert.assertEquals("Staff Group %s".formatted(group), true, evaluator.check(req));
        }

        for(var role : Role.regularRoles()) {
            var principal = new SubatoPrincipalBuilder().role(role).build(mapper);

            var req = new CheckPolicyRequest(principal, -1L, CoursesAction.COURSE_UPDATE, new HashMap<>());
            Assert.assertEquals("Role %s".formatted(role), true, evaluator.check(req));
        }

        // participant
        var principal = new SubatoPrincipalBuilder().participantOf(course).build(mapper);
        var req = new CheckPolicyRequest(principal, -1L, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(true, evaluator.check(req));

        // guest
        req = new CheckPolicyRequest(null, -1L, CoursesAction.COURSE_UPDATE, new HashMap<>());
        Assert.assertEquals(true, evaluator.check(req));
    }
}
